unit FntCombo;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls;

type
  PFontRecord = ^TFontRecord;
  TFontRecord = record
     FontName:string;
     CharSet : byte;
     IsTrueType:boolean;
  end;

// changed by S.Kawamura (TComboBox->TCustomComboBox)
  TFontComboBoxEx = class(TCustomComboBox)
  private
// added by S.Kawamura 1997.6.23
    TrueTypeBMP: TBitmap;
    NotTrueTypeBMP: TBitmap;
    FFont : TFont;
    FFontData : array of TFontRecord;
    FIncludeShiftJisCharset : boolean;
    FIncludeNotShiftJisCharset : boolean;
    procedure CMFontChanged(var Message: TMessage); message CM_FONTCHANGED;
    procedure ResetItemHeight;
// moved by S.Kawamura 1997.6.23
// changed by S.Kawamura 2002.12.14
    function FontNameToTFont : TFont;
    procedure TFontToFontName(AFont: TFont);
    procedure SetFontStructure(FontStr: string);
    procedure SetIncludeShiftJisCharset(Value : boolean);
    procedure SetIncludeNotShiftJisCharset(Value : boolean);

// added by S.Kawamura 1997.6.23
  protected
    procedure CreateWnd; override;
    procedure DrawItem(Index: Integer; Rect: TRect;
              State: TOwnerDrawState); override;
    procedure Click; override;
  public
    constructor Create(AOwner:TComponent); override;
    destructor Destroy; override;
    procedure ReBuildList;
  published
// added by S.Kawamura 2002.12.14
    property Anchors;
    property Constraints;
    property ImeMode;
    property ImeName;
// added by S.Kawamura 1997.6.23
    property Color;
    property Ctl3D;
    property DragMode;
    property DragCursor;
    property Enabled;
    property Font;
// added by S.Kawamura 1997.6.23
// public->published and string->TFont   2002.12.13
    property SelectedFont : TFont read FontNameToTFont
                                  write TFontToFontName;
    property IncludeShiftJisCharset : boolean
        read FIncludeShiftJisCharset
        write SetIncludeShiftJisCharset;
    property IncludeNotShiftJisCharset : boolean
        read FIncludeNotShiftJisCharset
        write SetIncludeNotShiftJisCharset;
    property ParentColor;
    property ParentCtl3D;
    property ParentFont;
    property ParentShowHint;
    property PopupMenu;
    property ShowHint;
    property TabOrder;
    property TabStop;
    property Visible;
    property OnChange;
    property OnClick;
    property OnDblClick;
    property OnDragDrop;
    property OnDragOver;
    property OnDropDown;
    property OnEndDrag;
    property OnEnter;
    property OnExit;
    property OnKeyDown;
    property OnKeyPress;
    property OnKeyUp;
    property OnStartDrag;
    property OnContextPopup;
  end;

procedure Register;

implementation

{$R FntCombo.res}


function GetItemHeight(Font: TFont): Integer;
var
  DC: HDC;
  SaveFont: HFont;
  Metrics: TTextMetric;
begin
  DC := GetDC(0);
  SaveFont := SelectObject(DC, Font.Handle);
  GetTextMetrics(DC, Metrics);
  SelectObject(DC, SaveFont);
  ReleaseDC(0, DC);
  Result := Metrics.tmHeight;
end;

// changed by S.Kawamura 1997.6.24
function EnumFontsProc(var LogFont: TLogFont; var TextMetric: TTextMetric;
  FontType: Integer; Data: Pointer): Integer; stdcall;
var AFontRecord:PFontRecord;
begin
  New(AFontRecord);
  AFontRecord^.FontName := LogFont.lfFaceName;
  AFontRecord^.CharSet := LogFont.lfCharSet;
  if (FontType>=TRUETYPE_FONTTYPE) then
     AFontRecord^.IsTrueType:=True
  else
     AFontRecord^.IsTrueType:=False;
  TList(Data).Add(AFontRecord);
  Result:= 1;
end;

// added by S.Kawamura 1997.6.23
constructor TFontComboBoxEx.Create(AOwner:TComponent);
begin
  inherited Create(AOwner);
  Style:=csOwnerDrawFixed;
  sorted := true;
  FFont := TFont.Create;
  FIncludeShiftJisCharset := true;
  FIncludeNotShiftJisCharset := true;
  TrueTypeBMP := TBitmap.Create;
  TrueTypeBMP.Handle := LoadBitmap(HInstance, 'TRUETYPE');
  NotTrueTypeBMP := TBitmap.Create;
  NotTrueTypeBMP.Handle := LoadBitmap(HInstance, 'NOTTT');
  FFont.Assign(Self.Font);
  ResetItemHeight;
end;

destructor TFontComboBoxEx.Destroy;
begin
  TrueTypeBMP.Free;
  NotTrueTypeBMP.Free;
  FFont.Free;
  inherited Destroy;
end;

procedure TFontComboBoxEx.ReBuildList;
var
  DC: HDC;
  FontList:TList;
  i: integer;
  AFontRecord:PFontRecord;
begin
  Clear;
  DC:= GetDC(0);
  FontList:=TList.Create;
  try
     EnumFonts(DC, nil, @EnumFontsProc, Pointer(FontList));
     SetLength(FFontData, FontList.Count);
     for i:=0 to (FontList.Count -1) do
     begin
          AFontRecord:=FontList.Items[i];
          FFontData[i].CharSet := AFontRecord^.CharSet;
          FFontData[i].FontName := AFontRecord^.FontName;
          FFontData[i].IsTrueType := AFontRecord^.IsTrueType;
          if not (((FIncludeShiftJisCharset = false) and
                   (FFontData[i].CharSet = SHIFTJIS_CHARSET)) or
                  ((FIncludeNotShiftJisCharset = false) and
                   (FFontData[i].CharSet <> SHIFTJIS_CHARSET))) then
          begin
            if AFontRecord^.IsTrueType then
               Items.AddObject(FFontData[i].FontName, TrueTypeBMP)
            else
               Items.AddObject(FFontData[i].FontName, NotTrueTypeBMP);
          end;
     end;
  finally
     for i := 0 to (FontList.Count - 1) do
       Dispose(FontList.Items[i]);
     FontList.Free;
  end;
  ReleaseDC(0, DC);

  i:=0;
  while i<Items.Count do
  begin
       if Items[i]=FFont.Name then
       begin
            ItemIndex := i;
            i:=Items.Count+1;
            Change;
       end;
       inc(i);
  end;
  if i=Items.Count then  // Ȃ
  begin
      i:=0;
      while i<Items.Count do
      begin
           if (Items[i]='Arial') or
              (Items[i]='lr SVbN') then
           begin
                ItemIndex := i;
                i:=Items.Count+1;
                Change;
           end;
           inc(i);
      end;
  end;
end;

procedure TFontComboBoxEx.CreateWnd;
begin
  inherited CreateWnd;
  ReBuildList;
  SetFontStructure(FFont.Name);
end;

procedure TFontComboBoxEx.DrawItem(Index: Integer; Rect: TRect;
  State: TOwnerDrawState);
var
  Bitmap: TBitmap;
  bmpWidth: Integer;
begin
  with Canvas do
  begin
    FillRect(Rect);
    bmpWidth  := 16;
    Bitmap := TBitmap(Items.Objects[Index]);
    if Bitmap <> nil then
    begin
      bmpWidth := Bitmap.Width;
      BrushCopy(Bounds(Rect.Left + 2,
               (Rect.Top + Rect.Bottom - Bitmap.Height) div 2,
               Bitmap.Width, Bitmap.Height),
               Bitmap, Bounds(0, 0, Bitmap.Width, Bitmap.Height),
               Bitmap.Canvas.Pixels[0, Bitmap.Height - 1]);
    end;
     { uses DrawText instead of TextOut in order to get clipping against
       the combo box button   }
    Rect.Left := Rect.Left + bmpWidth + 6;
    if not Enabled then Canvas.Font.Color := clGrayText;
    DrawText(Canvas.Handle, PChar(Items[Index]), -1, Rect,
             DT_SINGLELINE or DT_VCENTER or DT_NOPREFIX);
  end;
  SetFontStructure(Items[Index]);
end;

procedure TFontComboBoxEx.CMFontChanged(var Message: TMessage);
begin
  inherited;
  ResetItemHeight;
  RecreateWnd;
end;

procedure TFontComboBoxEx.Click;
begin
  inherited Click;
  if ItemIndex >= 0 then
    SetFontStructure(Items[ItemIndex]);
end;

procedure TFontComboBoxEx.ResetItemHeight;
var
  nuHeight: Integer;
begin
  nuHeight :=  GetItemHeight(Font);
  if nuHeight < (TrueTypeBMP.Height) then nuHeight := TrueTypeBmp.Height;
  ItemHeight := nuHeight;
end;

function TFontComboBoxEx.FontNameToTFont : TFont;
begin
   Result := FFont;
end;

procedure TFontComboBoxEx.TFontToFontName(AFont: TFont);
begin
   FFont.Assign(AFont);
   RebuildList;
end;

procedure TFontComboBoxEx.SetFontStructure(FontStr: string);
var i : integer;
begin
  i := 0;
  while FFontData[i].FontName<>FontStr do
    inc(i);
  FFont.Name:=FontStr;
  FFont.Charset := FFontData[i].CharSet;
end;

procedure TFontComboBoxEx.SetIncludeShiftJisCharset(Value : boolean);
begin
  if FIncludeShiftJisCharset <> Value then
  begin
     FIncludeShiftJisCharset := Value;
     RebuildList;
  end;
end;

procedure TFontComboBoxEx.SetIncludeNotShiftJisCharset(Value : boolean);
begin
  if FIncludeNotShiftJisCharset <> Value then
  begin
     FIncludeNotShiftJisCharset := Value;
     RebuildList;
  end;
end;

{ R|[lg̓o^ }
procedure Register;
begin
// changed by S.Kawamura 1997.6.23 (Samples->System)
  RegisterComponents('System', [TFontComboBoxEx]);
end;

end.
